"""Python abstract syntax node definitions

This file is automatically generated by Tools/compiler/astgen.py
"""
from compiler.consts import CO_VARARGS, CO_VARKEYWORDS

def flatten(seq):
    l = []
    for elt in seq:
        t = type(elt)
        if t is tuple or t is list:
            for elt2 in flatten(elt):
                l.append(elt2)
        else:
            l.append(elt)
    return l

def flatten_nodes(seq):
    return [n for n in flatten(seq) if isinstance(n, Node)]

def indent(s):
    return s.replace("\n", "\n\t")

def decode_function(node):
    star = (node.flags & 4 != 0) and 1 or 0
    dstar = (node.flags & 8 != 0) and 1 or 0
    argnames = node.argnames[:]

    # Add stars to star and dstar parameters.

    if star:
        argnames[-dstar-star] = "*%s" % argnames[-dstar-star]
    if dstar:
        argnames[-dstar] = "**%s" % argnames[-dstar]

    # Map defaults to parameters.

    defaults = [None] * (len(node.argnames) - star - dstar - len(node.defaults)) + list(node.defaults) + [None] * (star + dstar)
    return [(default and "%s=%s" % (argname, default) or argname) for (argname, default) in zip(argnames, defaults)]

nodes = {}

class AttributeUser:

    "Annotation-related node."

    def __init__(self):

        # Name and usage observations.

        self._attrnames = None
        self._attrcombined = None
        self._attrmerged = None

        # Related nodes.

        self._attrbranches = None
        self._attrcontributors = None
        self._attrdefs = None

        # Deductions.

        self._attrtypes = None
        self._attrspecifictypes = None

class OperatorUser:

    "Operator-related node."

    def __init__(self):
        self._module = None

class Node(AttributeUser):

    "Abstract base class for ast nodes."

    def getChildren(self):
        pass # implemented by subclasses

    def __iter__(self):
        for n in self.getChildren():
            yield n

    def asList(self): # for backwards compatibility
        return self.getChildren()

    def getChildNodes(self):
        pass # implemented by subclasses

    def visit(self, visitor, *args):
        visitor.default(self, *args)

class EmptyNode(Node):
    pass

class Expression(Node):
    # Expression is an artificial node class to support "eval"
    nodes["expression"] = "Expression"
    def __init__(self, node):
        Node.__init__(self)
        self.node = node

    def getChildren(self):
        return self.node,

    def getChildNodes(self):
        return self.node,

    def __repr__(self):
        return "Expression(%r)" % (self.node,)

    def __str__(self):
        return str(self.node)

    def visit(self, visitor, *args):
        return visitor.visitExpression(self, *args)

class Add(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "Add((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s + %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitAdd(self, *args)

class And(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "And(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % " and ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitAnd(self, *args)

class AssAttr(Node):
    def __init__(self, expr, attrname, flags, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.attrname = attrname
        self.flags = flags
        self.lineno = lineno

        # Additional annotations.

        self._expr = None
        self._attr = None
        self._attrusers = None
        self._username = None

    def getChildren(self):
        return self.expr, self.attrname, self.flags

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "AssAttr(%r, %r, %r)" % (self.expr, self.attrname, self.flags)

    def __str__(self):
        if self.flags == "OP_DELETE":
            return "del %s.%s" % (self.expr, self.attrname)
        else:
            return "%s.%s" % (self.expr, self.attrname)

    def visit(self, visitor, *args):
        return visitor.visitAssAttr(self, *args)

class AssList(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "AssList(%r)" % (self.nodes,)

    def __str__(self):
        return "[%s]" % ", ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitAssList(self, *args)

class AssName(Node):
    def __init__(self, name, flags, lineno=None):
        Node.__init__(self)
        self.name = name
        self.flags = flags
        self.lineno = lineno

    def getChildren(self):
        return self.name, self.flags

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "AssName(%r, %r)" % (self.name, self.flags)

    def __str__(self):
        if self.flags == "OP_DELETE":
            return "del %s" % self.name
        else:
            return str(self.name)

    def visit(self, visitor, *args):
        return visitor.visitAssName(self, *args)

class AssTuple(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "AssTuple(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % ", ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitAssTuple(self, *args)

class Assert(Node):
    def __init__(self, test, fail, lineno=None):
        Node.__init__(self)
        self.test = test
        self.fail = fail
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.test)
        children.append(self.fail)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.test)
        if self.fail is not None:
            nodelist.append(self.fail)
        return tuple(nodelist)

    def __repr__(self):
        return "Assert(%r, %r)" % (self.test, self.fail)

    def __str__(self):
        return "assert %s%s" % (self.test, self.fail and ", %s" % self.fail or "")

    def visit(self, visitor, *args):
        return visitor.visitAssert(self, *args)

class Assign(Node):
    def __init__(self, nodes, expr, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.extend(flatten(self.nodes))
        children.append(self.expr)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        nodelist.append(self.expr)
        return tuple(nodelist)

    def __repr__(self):
        return "Assign(%r, %r)" % (self.nodes, self.expr)

    def __str__(self):
        return "%s = %s" % (", ".join(map(str, self.nodes)), self.expr)

    def visit(self, visitor, *args):
        return visitor.visitAssign(self, *args)

class AugAssign(Node, OperatorUser):
    def __init__(self, node, op, expr, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.node = node
        self.op = op
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.node, self.op, self.expr

    def getChildNodes(self):
        return self.node, self.expr

    def __repr__(self):
        return "AugAssign(%r, %r, %r)" % (self.node, self.op, self.expr)

    def __str__(self):
        return "%s %s %s" % (self.node, self.op, self.expr)

    def visit(self, visitor, *args):
        return visitor.visitAugAssign(self, *args)

class Backquote(Node):
    def __init__(self, expr, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.expr,

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "Backquote(%r)" % (self.expr,)

    def __str__(self):
        return "`%s`" % self.expr

    def visit(self, visitor, *args):
        return visitor.visitBackquote(self, *args)

class Bitand(Node, OperatorUser):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Bitand(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % " & ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitBitand(self, *args)

class Bitor(Node, OperatorUser):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Bitor(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % " | ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitBitor(self, *args)

class Bitxor(Node, OperatorUser):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Bitxor(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % " ^ ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitBitxor(self, *args)

class Break(Node):
    def __init__(self, lineno=None):
        Node.__init__(self)
        self.lineno = lineno

    def getChildren(self):
        return ()

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Break()"

    def __str__(self):
        return "break"

    def visit(self, visitor, *args):
        return visitor.visitBreak(self, *args)

class CallFunc(Node):
    def __init__(self, node, args, star_args = None, dstar_args = None, lineno=None):
        Node.__init__(self)
        self.node = node
        self.args = args
        self.star_args = star_args
        self.dstar_args = dstar_args
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.node)
        children.extend(flatten(self.args))
        children.append(self.star_args)
        children.append(self.dstar_args)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.node)
        nodelist.extend(flatten_nodes(self.args))
        if self.star_args is not None:
            nodelist.append(self.star_args)
        if self.dstar_args is not None:
            nodelist.append(self.dstar_args)
        return tuple(nodelist)

    def __repr__(self):
        return "CallFunc(%r, %r, %r, %r)" % (self.node, self.args, self.star_args, self.dstar_args)

    def __str__(self):
        star_args = self.star_args and ["*%s" % self.star_args] or []
        dstar_args = self.dstar_args and ["**%s" % self.dstar_args] or []
        return "%s(%s)" % (self.node, ", ".join(map(str, self.args + star_args + dstar_args)))

    def visit(self, visitor, *args):
        return visitor.visitCallFunc(self, *args)

class Class(Node):
    def __init__(self, name, bases, doc, code, decorators = None, lineno=None):
        Node.__init__(self)
        self.name = name
        self.bases = bases
        self.doc = doc
        self.code = code
        self.decorators = decorators
        self.lineno = lineno

        # Additional annotations.

        self.unit = None

    def getChildren(self):
        children = []
        children.append(self.name)
        children.extend(flatten(self.bases))
        children.append(self.doc)
        children.append(self.code)
        children.append(self.decorators)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.bases))
        nodelist.append(self.code)
        if self.decorators is not None:
            nodelist.append(self.decorators)
        return tuple(nodelist)

    def __repr__(self):
        return "Class(%r, %r, %r, %r, %r)" % (self.name, self.bases, self.doc, self.code, self.decorators)

    def __str__(self):
        return "%sclass %s%s:%s%s" % (
            self.decorators and "%s\n" % "\n".join([("@%s" % decorator) for decorator in self.decorators]) or "",
            self.name,
            self.bases and "(%s)" % ", ".join(map(str, self.bases)) or "",
            indent(self.doc and "\n%r" % self.doc or ""),
            indent("\n%s" % self.code)
            )

    def visit(self, visitor, *args):
        return visitor.visitClass(self, *args)

class Compare(Node, OperatorUser):
    def __init__(self, expr, ops, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.expr = expr
        self.ops = ops
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.extend(flatten(self.ops))
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        nodelist.extend(flatten_nodes(self.ops))
        return tuple(nodelist)

    def __repr__(self):
        return "Compare(%r, %r)" % (self.expr, self.ops)

    def __str__(self):
        return "%s %s" % (self.expr, " ".join([("%s %s" % op) for op in self.ops]))

    def visit(self, visitor, *args):
        return visitor.visitCompare(self, *args)

class Const(Node):
    def __init__(self, value, lineno=None):
        Node.__init__(self)
        self.value = value
        self.lineno = lineno

    def getChildren(self):
        return self.value,

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Const(%r)" % (self.value,)

    def __str__(self):
        return repr(self.value)

    def visit(self, visitor, *args):
        return visitor.visitConst(self, *args)

class Continue(Node):
    def __init__(self, lineno=None):
        Node.__init__(self)
        self.lineno = lineno

    def getChildren(self):
        return ()

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Continue()"

    def __str__(self):
        return "continue"

    def visit(self, visitor, *args):
        return visitor.visitContinue(self, *args)

class Decorators(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Decorators(%r)" % (self.nodes,)

    def __str__(self):
        return "\n".join([("@%s" % node) for node in self.nodes])

    def visit(self, visitor, *args):
        return visitor.visitDecorators(self, *args)

class Dict(Node):
    def __init__(self, items, lineno=None):
        Node.__init__(self)
        self.items = items
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.items))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.items))
        return tuple(nodelist)

    def __repr__(self):
        return "Dict(%r)" % (self.items,)

    def __str__(self):
        return "{%s}" % ", ".join([("%s : %s" % (key, value)) for (key, value) in self.items])

    def visit(self, visitor, *args):
        return visitor.visitDict(self, *args)

class Discard(Node):
    def __init__(self, expr, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.expr,

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "Discard(%r)" % (self.expr,)

    def __str__(self):
        return str(self.expr)

    def visit(self, visitor, *args):
        return visitor.visitDiscard(self, *args)

class Div(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "Div((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s / %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitDiv(self, *args)

class Ellipsis(Node):
    def __init__(self, lineno=None):
        Node.__init__(self)
        self.lineno = lineno

    def getChildren(self):
        return ()

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Ellipsis()"

    def __str__(self):
        return "..."

    def visit(self, visitor, *args):
        return visitor.visitEllipsis(self, *args)

class Exec(Node):
    def __init__(self, expr, locals, globals, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.locals = locals
        self.globals = globals
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.append(self.locals)
        children.append(self.globals)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        if self.locals is not None:
            nodelist.append(self.locals)
        if self.globals is not None:
            nodelist.append(self.globals)
        return tuple(nodelist)

    def __repr__(self):
        return "Exec(%r, %r, %r)" % (self.expr, self.locals, self.globals)

    def __str__(self):
        return "exec %s%s%s" % (self.expr, self.locals and ", %s" % self.locals or "",
            self.globals and ", %s" % self.globals or "")

    def visit(self, visitor, *args):
        return visitor.visitExec(self, *args)

class FloorDiv(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "FloorDiv((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s // %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitFloorDiv(self, *args)

class For(Node):
    def __init__(self, assign, list, body, else_, lineno=None):
        Node.__init__(self)
        self.assign = assign
        self.list = list
        self.body = body
        self.else_ = else_
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.assign)
        children.append(self.list)
        children.append(self.body)
        children.append(self.else_)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.assign)
        nodelist.append(self.list)
        nodelist.append(self.body)
        if self.else_ is not None:
            nodelist.append(self.else_)
        return tuple(nodelist)

    def __repr__(self):
        return "For(%r, %r, %r, %r)" % (self.assign, self.list, self.body, self.else_)

    def __str__(self):
        return "for %s in %s:%s%s" % (
            self.assign, self.list,
            indent("\n%s" % self.body),
            self.else_ and "\nelse:%s" % indent("\n%s" % self.else_) or ""
            )

    def visit(self, visitor, *args):
        return visitor.visitFor(self, *args)

class From(Node):
    def __init__(self, modname, names, level, lineno=None):
        Node.__init__(self)
        self.modname = modname
        self.names = names
        self.level = level
        self.lineno = lineno

    def getChildren(self):
        return self.modname, self.names, self.level

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "From(%r, %r, %r)" % (self.modname, self.names, self.level)

    def __str__(self):
        return "from %s import %s" % (self.modname,
            ", ".join([(alias and "%s as %s" % (name, alias) or name) for (name, alias) in self.names]))

    def visit(self, visitor, *args):
        return visitor.visitFrom(self, *args)

class Function(Node):
    def __init__(self, decorators, name, argnames, defaults, flags, doc, code, lineno=None):
        Node.__init__(self)
        self.decorators = decorators
        self.name = name
        self.argnames = argnames
        self.defaults = defaults
        self.flags = flags
        self.doc = doc
        self.code = code
        self.lineno = lineno
        self.varargs = self.kwargs = None
        if flags & CO_VARARGS:
            self.varargs = 1
        if flags & CO_VARKEYWORDS:
            self.kwargs = 1

        # Additional annotations.

        self.unit = None

    def getChildren(self):
        children = []
        children.append(self.decorators)
        children.append(self.name)
        children.append(self.argnames)
        children.extend(flatten(self.defaults))
        children.append(self.flags)
        children.append(self.doc)
        children.append(self.code)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        if self.decorators is not None:
            nodelist.append(self.decorators)
        nodelist.extend(flatten_nodes(self.defaults))
        nodelist.append(self.code)
        return tuple(nodelist)

    def __repr__(self):
        return "Function(%r, %r, %r, %r, %r, %r, %r)" % (self.decorators, self.name, self.argnames, self.defaults, self.flags, self.doc, self.code)

    def __str__(self):
        parameters = decode_function(self)

        return "%sdef %s(%s):%s%s" % (
            self.decorators and "%s\n" % "\n".join([("@%s" % decorator) for decorator in self.decorators]) or "",
            self.name,
            ", ".join(parameters),
            indent(self.doc and "\n%r" % self.doc or ""),
            indent("\n%s" % self.code)
            )

    def visit(self, visitor, *args):
        return visitor.visitFunction(self, *args)

class GenExpr(Node):
    def __init__(self, code, lineno=None):
        Node.__init__(self)
        self.code = code
        self.lineno = lineno
        self.argnames = ['.0']
        self.varargs = self.kwargs = None

    def getChildren(self):
        return self.code,

    def getChildNodes(self):
        return self.code,

    def __repr__(self):
        return "GenExpr(%r)" % (self.code,)

    def __str__(self):
        return str(self.code)

    def visit(self, visitor, *args):
        return visitor.visitGenExpr(self, *args)

class GenExprFor(Node):
    def __init__(self, assign, iter, ifs, lineno=None):
        Node.__init__(self)
        self.assign = assign
        self.iter = iter
        self.ifs = ifs
        self.lineno = lineno
        self.is_outmost = False

    def getChildren(self):
        children = []
        children.append(self.assign)
        children.append(self.iter)
        children.extend(flatten(self.ifs))
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.assign)
        nodelist.append(self.iter)
        nodelist.extend(flatten_nodes(self.ifs))
        return tuple(nodelist)

    def __repr__(self):
        return "GenExprFor(%r, %r, %r)" % (self.assign, self.iter, self.ifs)

    def __str__(self):
        return "for %s in %s%s" % (
            self.assign, self.iter,
            self.ifs and " ".join(map(str, self.ifs)) or ""
            )

    def visit(self, visitor, *args):
        return visitor.visitGenExprFor(self, *args)

class GenExprIf(Node):
    def __init__(self, test, lineno=None):
        Node.__init__(self)
        self.test = test
        self.lineno = lineno

    def getChildren(self):
        return self.test,

    def getChildNodes(self):
        return self.test,

    def __repr__(self):
        return "GenExprIf(%r)" % (self.test,)

    def __str__(self):
        return "if %s" % self.test

    def visit(self, visitor, *args):
        return visitor.visitGenExprIf(self, *args)

class GenExprInner(Node):
    def __init__(self, expr, quals, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.quals = quals
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.extend(flatten(self.quals))
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        nodelist.extend(flatten_nodes(self.quals))
        return tuple(nodelist)

    def __repr__(self):
        return "GenExprInner(%r, %r)" % (self.expr, self.quals)

    def __str__(self):
        return "%s %s" % (self.expr, " ".join(map(str, self.quals)))

    def visit(self, visitor, *args):
        return visitor.visitGenExprInner(self, *args)

class Getattr(Node):
    def __init__(self, expr, attrname, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.attrname = attrname
        self.lineno = lineno

        # Additional annotations.

        self._expr = None
        self._attr = None
        self._attrusers = None
        self._username = None

    def getChildren(self):
        return self.expr, self.attrname

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "Getattr(%r, %r)" % (self.expr, self.attrname)

    def __str__(self):
        return "%s.%s" % (self.expr, self.attrname)

    def visit(self, visitor, *args):
        return visitor.visitGetattr(self, *args)

class Global(Node):
    def __init__(self, names, lineno=None):
        Node.__init__(self)
        self.names = names
        self.lineno = lineno

    def getChildren(self):
        return self.names,

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Global(%r)" % (self.names,)

    def __str__(self):
        return "global %s" % ", ".join(map(str, self.names))

    def visit(self, visitor, *args):
        return visitor.visitGlobal(self, *args)

class If(Node):
    def __init__(self, tests, else_, lineno=None):
        Node.__init__(self)
        self.tests = tests
        self.else_ = else_
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.extend(flatten(self.tests))
        children.append(self.else_)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.tests))
        if self.else_ is not None:
            nodelist.append(self.else_)
        return tuple(nodelist)

    def __repr__(self):
        return "If(%r, %r)" % (self.tests, self.else_)

    def __str__(self):
        tests = [("%sif %s:%s" % (i > 0 and "el" or "", test, indent("\n%s" % body))) for (i, (test, body)) in enumerate(self.tests)]
        return "%s%s" % (
            "\n".join(tests),
            self.else_ and "\nelse:%s" % indent("\n%s" % self.else_) or ""
            )

    def visit(self, visitor, *args):
        return visitor.visitIf(self, *args)

class IfExp(Node):
    def __init__(self, test, then, else_, lineno=None):
        Node.__init__(self)
        self.test = test
        self.then = then
        self.else_ = else_
        self.lineno = lineno

    def getChildren(self):
        return self.test, self.then, self.else_

    def getChildNodes(self):
        return self.test, self.then, self.else_

    def __repr__(self):
        return "IfExp(%r, %r, %r)" % (self.test, self.then, self.else_)

    def __str__(self):
        return "%s if %s else %s" % (self.then, self.test, self.else_)

    def visit(self, visitor, *args):
        return visitor.visitIfExp(self, *args)

class Import(Node):
    def __init__(self, names, lineno=None):
        Node.__init__(self)
        self.names = names
        self.lineno = lineno

    def getChildren(self):
        return self.names,

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Import(%r)" % (self.names,)

    def __str__(self):
        return "import %s" % (
            ", ".join([(alias and "%s as %s" % (name, alias) or name) for (name, alias) in self.names]))

    def visit(self, visitor, *args):
        return visitor.visitImport(self, *args)

class Invert(Node, OperatorUser):
    def __init__(self, expr, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.expr,

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "Invert(%r)" % (self.expr,)

    def __str__(self):
        return "~%s" % self.expr

    def visit(self, visitor, *args):
        return visitor.visitInvert(self, *args)

class Keyword(Node):
    def __init__(self, name, expr, lineno=None):
        Node.__init__(self)
        self.name = name
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.name, self.expr

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "Keyword(%r, %r)" % (self.name, self.expr)

    def __str__(self):
        return "%s=%s" % (self.name, self.expr)

    def visit(self, visitor, *args):
        return visitor.visitKeyword(self, *args)

class Lambda(Node):
    def __init__(self, argnames, defaults, flags, code, lineno=None):
        Node.__init__(self)
        self.argnames = argnames
        self.defaults = defaults
        self.flags = flags
        self.code = code
        self.lineno = lineno
        self.varargs = self.kwargs = None
        if flags & CO_VARARGS:
            self.varargs = 1
        if flags & CO_VARKEYWORDS:
            self.kwargs = 1

    def getChildren(self):
        children = []
        children.append(self.argnames)
        children.extend(flatten(self.defaults))
        children.append(self.flags)
        children.append(self.code)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.defaults))
        nodelist.append(self.code)
        return tuple(nodelist)

    def __repr__(self):
        return "Lambda(%r, %r, %r, %r)" % (self.argnames, self.defaults, self.flags, self.code)

    def __str__(self):
        parameters = decode_function(self)
        return "lambda %s: %s" % (", ".join(parameters), self.code)

    def visit(self, visitor, *args):
        return visitor.visitLambda(self, *args)

class LeftShift(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "LeftShift((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s << %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitLeftShift(self, *args)

class List(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "List(%r)" % (self.nodes,)

    def __str__(self):
        return "[%s]" % ", ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitList(self, *args)

class ListComp(Node):
    def __init__(self, expr, quals, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.quals = quals
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.extend(flatten(self.quals))
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        nodelist.extend(flatten_nodes(self.quals))
        return tuple(nodelist)

    def __repr__(self):
        return "ListComp(%r, %r)" % (self.expr, self.quals)

    def __str__(self):
        return "[%s %s]" % (self.expr, " ".join(map(str, self.quals)))

    def visit(self, visitor, *args):
        return visitor.visitListComp(self, *args)

class ListCompFor(Node):
    def __init__(self, assign, list, ifs, lineno=None):
        Node.__init__(self)
        self.assign = assign
        self.list = list
        self.ifs = ifs
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.assign)
        children.append(self.list)
        children.extend(flatten(self.ifs))
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.assign)
        nodelist.append(self.list)
        nodelist.extend(flatten_nodes(self.ifs))
        return tuple(nodelist)

    def __repr__(self):
        return "ListCompFor(%r, %r, %r)" % (self.assign, self.list, self.ifs)

    def __str__(self):
        return "for %s in %s%s" % (
            self.assign, self.list,
            self.ifs and " ".join(map(str, self.ifs)) or ""
            )

    def visit(self, visitor, *args):
        return visitor.visitListCompFor(self, *args)

class ListCompIf(Node):
    def __init__(self, test, lineno=None):
        Node.__init__(self)
        self.test = test
        self.lineno = lineno

    def getChildren(self):
        return self.test,

    def getChildNodes(self):
        return self.test,

    def __repr__(self):
        return "ListCompIf(%r)" % (self.test,)

    def __str__(self):
        return "if %s" % self.test

    def visit(self, visitor, *args):
        return visitor.visitListCompIf(self, *args)

class Mod(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "Mod((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s %% %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitMod(self, *args)

class Module(Node):
    def __init__(self, doc, node, lineno=None):
        Node.__init__(self)
        self.doc = doc
        self.node = node
        self.lineno = lineno

        # Additional annotations.

        self.unit = None

    def getChildren(self):
        return self.doc, self.node

    def getChildNodes(self):
        return self.node,

    def __repr__(self):
        return "Module(%r, %r)" % (self.doc, self.node)

    def __str__(self):
        return "%s%s" % (self.doc and "%r\n" % self.doc or "", self.node)

    def visit(self, visitor, *args):
        return visitor.visitModule(self, *args)

class Mul(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "Mul((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s * %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitMul(self, *args)

class Name(Node):
    def __init__(self, name, lineno=None):
        Node.__init__(self)
        self.name = name
        self.lineno = lineno

        # Additional annotations.

        self._scope = None
        self._attr = None
        self._attrusers = None
        self._username = None

    def getChildren(self):
        return self.name,

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Name(%r)" % (self.name,)

    def __str__(self):
        return str(self.name)

    def visit(self, visitor, *args):
        return visitor.visitName(self, *args)

class Not(Node):
    def __init__(self, expr, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.expr,

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "Not(%r)" % (self.expr,)

    def __str__(self):
        return "not %s" % self.expr

    def visit(self, visitor, *args):
        return visitor.visitNot(self, *args)

class Or(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Or(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % " or ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitOr(self, *args)

class Pass(Node):
    def __init__(self, lineno=None):
        Node.__init__(self)
        self.lineno = lineno

    def getChildren(self):
        return ()

    def getChildNodes(self):
        return ()

    def __repr__(self):
        return "Pass()"

    def __str__(self):
        return "pass"

    def visit(self, visitor, *args):
        return visitor.visitPass(self, *args)

class Power(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "Power((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s ** %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitPower(self, *args)

class Print(Node):
    def __init__(self, nodes, dest, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.dest = dest
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.extend(flatten(self.nodes))
        children.append(self.dest)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        if self.dest is not None:
            nodelist.append(self.dest)
        return tuple(nodelist)

    def __repr__(self):
        return "Print(%r, %r)" % (self.nodes, self.dest)

    def __str__(self):
        dest = self.dest and [">>%s" % self.dest] or []
        return "print %s," % ", ".join(map(str, dest + self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitPrint(self, *args)

class Printnl(Node):
    def __init__(self, nodes, dest, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.dest = dest
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.extend(flatten(self.nodes))
        children.append(self.dest)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        if self.dest is not None:
            nodelist.append(self.dest)
        return tuple(nodelist)

    def __repr__(self):
        return "Printnl(%r, %r)" % (self.nodes, self.dest)

    def __str__(self):
        dest = self.dest and [">>%s" % self.dest] or []
        return "print %s" % ", ".join(map(str, dest + self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitPrintnl(self, *args)

class Raise(Node):
    def __init__(self, expr1, expr2, expr3, lineno=None):
        Node.__init__(self)
        self.expr1 = expr1
        self.expr2 = expr2
        self.expr3 = expr3
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr1)
        children.append(self.expr2)
        children.append(self.expr3)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        if self.expr1 is not None:
            nodelist.append(self.expr1)
        if self.expr2 is not None:
            nodelist.append(self.expr2)
        if self.expr3 is not None:
            nodelist.append(self.expr3)
        return tuple(nodelist)

    def __repr__(self):
        return "Raise(%r, %r, %r)" % (self.expr1, self.expr2, self.expr3)

    def __str__(self):
        args = self.expr1 and [self.expr1] or []
        args += self.expr2 and [self.expr2] or []
        args += self.expr3 and [self.expr3] or []
        return "raise %s" % ", ".join(map(str, args))

    def visit(self, visitor, *args):
        return visitor.visitRaise(self, *args)

class Return(Node):
    def __init__(self, value, lineno=None):
        Node.__init__(self)
        self.value = value
        self.lineno = lineno

    def getChildren(self):
        return self.value,

    def getChildNodes(self):
        return self.value,

    def __repr__(self):
        return "Return(%r)" % (self.value,)

    def __str__(self):
        return "return %s" % self.value

    def visit(self, visitor, *args):
        return visitor.visitReturn(self, *args)

class RightShift(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "RightShift((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s >> %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitRightShift(self, *args)

class Slice(Node, OperatorUser):
    def __init__(self, expr, flags, lower, upper, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.expr = expr
        self.flags = flags
        self.lower = lower
        self.upper = upper
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.append(self.flags)
        children.append(self.lower)
        children.append(self.upper)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        if self.lower is not None:
            nodelist.append(self.lower)
        if self.upper is not None:
            nodelist.append(self.upper)
        return tuple(nodelist)

    def __repr__(self):
        return "Slice(%r, %r, %r, %r)" % (self.expr, self.flags, self.lower, self.upper)

    def __str__(self):
        args = []
        args = self.lower is not None and [self.lower] or []
        args = self.upper is not None and [self.upper] or []
        return "%s[%s]" % (self.expr, ":".join(map(str, args)))

    def visit(self, visitor, *args):
        return visitor.visitSlice(self, *args)

class Sliceobj(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Sliceobj(%r)" % (self.nodes,)

    def __str__(self):
        return ":".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitSliceobj(self, *args)

class Stmt(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Stmt(%r)" % (self.nodes,)

    def __str__(self):
        return "\n".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitStmt(self, *args)

class Sub(Node, OperatorUser):
    def __init__(self, leftright, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.left = leftright[0]
        self.right = leftright[1]
        self.lineno = lineno

    def getChildren(self):
        return self.left, self.right

    def getChildNodes(self):
        return self.left, self.right

    def __repr__(self):
        return "Sub((%r, %r))" % (self.left, self.right)

    def __str__(self):
        return "(%s - %s)" % (self.left, self.right)

    def visit(self, visitor, *args):
        return visitor.visitSub(self, *args)

class Subscript(Node, OperatorUser):
    def __init__(self, expr, flags, subs, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.expr = expr
        self.flags = flags
        self.subs = subs
        self.lineno = lineno

        # Additional annotations.

        self._expr = None
        self._attr = None
        self._attrusers = None
        self._username = None

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.append(self.flags)
        children.extend(flatten(self.subs))
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        nodelist.extend(flatten_nodes(self.subs))
        return tuple(nodelist)

    def __repr__(self):
        return "Subscript(%r, %r, %r)" % (self.expr, self.flags, self.subs)

    def __str__(self):
        return "%s[%s]" % (self.expr, ",".join(map(str, self.subs)))

    def visit(self, visitor, *args):
        return visitor.visitSubscript(self, *args)

class TryExcept(Node):
    def __init__(self, body, handlers, else_, lineno=None):
        Node.__init__(self)
        self.body = body
        self.handlers = handlers
        self.else_ = else_
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.body)
        children.extend(flatten(self.handlers))
        children.append(self.else_)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.body)
        nodelist.extend(flatten_nodes(self.handlers))
        if self.else_ is not None:
            nodelist.append(self.else_)
        return tuple(nodelist)

    def __repr__(self):
        return "TryExcept(%r, %r, %r)" % (self.body, self.handlers, self.else_)

    def __str__(self):
        handlers = [
            ("\nexcept%s%s:%s" % (spec and " %s" % spec or "", assign and ", %s" % assign or "", indent("\n%s" % statement)))
            for (spec, assign, statement) in self.handlers
            ]

        return "try:%s%s%s" % (
            indent("\n%s" % self.body),
            "".join(handlers),
            self.else_ and "\nelse:%s" % indent("\n%s" % self.else_) or ""
            )

    def visit(self, visitor, *args):
        return visitor.visitTryExcept(self, *args)

class TryFinally(Node):
    def __init__(self, body, final, lineno=None):
        Node.__init__(self)
        self.body = body
        self.final = final
        self.lineno = lineno

    def getChildren(self):
        return self.body, self.final

    def getChildNodes(self):
        return self.body, self.final

    def __repr__(self):
        return "TryFinally(%r, %r)" % (self.body, self.final)

    def __str__(self):
        return "try:%s\nfinally:%s" % (
            indent("\n%s" % self.body),
            indent("\n%s" % self.final)
            )

    def visit(self, visitor, *args):
        return visitor.visitTryFinally(self, *args)

class Tuple(Node):
    def __init__(self, nodes, lineno=None):
        Node.__init__(self)
        self.nodes = nodes
        self.lineno = lineno

    def getChildren(self):
        return tuple(flatten(self.nodes))

    def getChildNodes(self):
        nodelist = []
        nodelist.extend(flatten_nodes(self.nodes))
        return tuple(nodelist)

    def __repr__(self):
        return "Tuple(%r)" % (self.nodes,)

    def __str__(self):
        return "(%s)" % ", ".join(map(str, self.nodes))

    def visit(self, visitor, *args):
        return visitor.visitTuple(self, *args)

class UnaryAdd(Node, OperatorUser):
    def __init__(self, expr, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.expr,

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "UnaryAdd(%r)" % (self.expr,)

    def __str__(self):
        return "+%s" % self.expr

    def visit(self, visitor, *args):
        return visitor.visitUnaryAdd(self, *args)

class UnarySub(Node, OperatorUser):
    def __init__(self, expr, lineno=None):
        Node.__init__(self)
        OperatorUser.__init__(self)
        self.expr = expr
        self.lineno = lineno

    def getChildren(self):
        return self.expr,

    def getChildNodes(self):
        return self.expr,

    def __repr__(self):
        return "UnarySub(%r)" % (self.expr,)

    def __str__(self):
        return "-%s" % self.expr

    def visit(self, visitor, *args):
        return visitor.visitUnarySub(self, *args)

class While(Node):
    def __init__(self, test, body, else_, lineno=None):
        Node.__init__(self)
        self.test = test
        self.body = body
        self.else_ = else_
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.test)
        children.append(self.body)
        children.append(self.else_)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.test)
        nodelist.append(self.body)
        if self.else_ is not None:
            nodelist.append(self.else_)
        return tuple(nodelist)

    def __repr__(self):
        return "While(%r, %r, %r)" % (self.test, self.body, self.else_)

    def __str__(self):
        return "while %s:%s%s" % (
            self.test,
            indent("\n%s" % self.body),
            self.else_ and "\nelse:%s" % indent("\n%s" % self.else_) or ""
            )

    def visit(self, visitor, *args):
        return visitor.visitWhile(self, *args)

class With(Node):
    def __init__(self, expr, vars, body, lineno=None):
        Node.__init__(self)
        self.expr = expr
        self.vars = vars
        self.body = body
        self.lineno = lineno

    def getChildren(self):
        children = []
        children.append(self.expr)
        children.append(self.vars)
        children.append(self.body)
        return tuple(children)

    def getChildNodes(self):
        nodelist = []
        nodelist.append(self.expr)
        if self.vars is not None:
            nodelist.append(self.vars)
        nodelist.append(self.body)
        return tuple(nodelist)

    def __repr__(self):
        return "With(%r, %r, %r)" % (self.expr, self.vars, self.body)

    def __str__(self):
        return "with %s%s:%s" % (
            self.expr,
            self.vars and " as %s" % ", ".join(map(str, self.vars)),
            indent("\n%s" % self.body),
            )

    def visit(self, visitor, *args):
        return visitor.visitWith(self, *args)

class Yield(Node):
    def __init__(self, value, lineno=None):
        Node.__init__(self)
        self.value = value
        self.lineno = lineno

    def getChildren(self):
        return self.value,

    def getChildNodes(self):
        return self.value,

    def __repr__(self):
        return "Yield(%r)" % (self.value,)

    def __str__(self):
        return "yield %s" % self.value

    def visit(self, visitor, *args):
        return visitor.visitYield(self, *args)

for name, obj in globals().items():
    if isinstance(obj, type) and issubclass(obj, Node):
        nodes[name.lower()] = obj
